@@ -73,7 +73,7 @@ impl ExternalSorter {
 
         // Write any items left in buffer, but only if we had at least 1 segment writen.
         // Otherwise we use the buffer itself to iterate from memory
-        let pass_through_queue = if buffer.len() > 0 && segments.len() > 0 {
+        let pass_through_queue = if !buffer.is_empty() && !segments.is_empty() {
             Self::sort_and_write_segment(&sort_dir, &mut segments, &mut buffer)?;
             None
         } else {
@@ -81,7 +81,7 @@ impl ExternalSorter {
             Some(VecDeque::from(buffer))
         };
 
-        Ok(SortedIterator::new(tempdir, pass_through_queue, segments))
+        SortedIterator::new(tempdir, pass_through_queue, segments)
     }
 
     fn sort_and_write_segment<T>(
@@ -136,25 +136,24 @@ impl<T: Sortable<T>> SortedIterator<T> {
         tempdir: Option<tempdir::TempDir>,
         pass_through_queue: Option<VecDeque<T>>,
         mut segments: Vec<File>,
-    ) -> SortedIterator<T> {
+    ) -> Result<SortedIterator<T>, Error> {
+        for segment in &mut segments {
+            segment.seek(SeekFrom::Start(0))?;
+        }
+
         let next_values = segments
             .iter_mut()
-            .map(|file| {
-                file.seek(SeekFrom::Start(0)).unwrap();
-                Self::read_item(file)
-            }).collect();
-
-        let segments = segments
-            .into_iter()
-            .map(|file| BufReader::new(file))
+            .map(|file| Self::read_item(file))
             .collect();
 
-        SortedIterator {
+        let segments = segments.into_iter().map(BufReader::new).collect();
+
+        Ok(SortedIterator {
             _tempdir: tempdir,
             pass_through_queue,
             segments,
             next_values,
-        }
+        })
     }
 
     fn read_item(file: &mut Read) -> Option<T> {
@@ -190,7 +189,7 @@ impl<T: Sortable<T>> Iterator for SortedIterator<T> {
 
         match smallest_idx {
             Some(idx) => {
-                let file = self.segments.get_mut(idx).unwrap();
+                let file = &mut self.segments[idx];
                 let value = self.next_values[idx].take().unwrap();
                 self.next_values[idx] = Self::read_item(file);
                 Some(value)
